/*************
 ColorLight.c
**************/

//***********
//* Includes
//***********
#include "ColorLight.h"
#include "ColorTable.h"

	
/**
* \brief
* Steuert das PWM Licht
*/
void PWMLight (void)
{
	unsigned char TableCount;
	unsigned int ZeitDifferenz;
	static unsigned int Red, Green, Blue, White; // muessen static sein
	
	ZeitDifferenz = TimeDiff (&RTC.Hour, &RTC.Min, &Alarm.Hour, &Alarm.Min);
	
	if (ZeitDifferenz > 5)
	{
		// Lichtintensitaet konstant erhoehen
		// ----------------------------------

		ZeitDifferenz -= 5;
		TableCount = 3 * (10 - ZeitDifferenz);
		if (RTC.Sec > 20)
		{
			TableCount++;
			if (RTC.Sec > 40)
			{
				TableCount++;
			}
		}
		Red = pgm_read_word(&Color_Table[TableCount]);
		Green = Red;
		Blue = Red;
		White = Red;
	}
	else
	{
		// Fading
		// ------

		if (ColorCnt > COLORCNTMAX)
		{
			// ColorCounter zuruecksetzen
			ColorCnt = 0;

			// Tabellencounter erhoehen
			IncChar(&RedTableCnt, 78, 0);
			IncChar(&GreenTableCnt, 78, 0);
			IncChar(&BlueTableCnt, 78, 0);
			
			// neuen PWM setzen
			Red = pgm_read_word(&Color_Table[RedTableCnt]);
			Green = pgm_read_word(&Color_Table[GreenTableCnt]);
			Blue = pgm_read_word(&Color_Table[BlueTableCnt]);
			White = 0; // Weisse LED aus
		}
	}

	REDPWM = Red;
	GREENPWM = Green;
	BLUEPWM = Blue;
	WHITEPWM = White;
}


/**
* \brief
* Schaltet die Timer fuer PWM wieder aus
*/
void UnIntiPWMPeripherie ( void )
{	
	// Timer0 Off
	TCCR0A=0x00;
	TCCR0B=0x00;
	TCNT0=0x00;
	OCR0A=0x00;
	OCR0B=0x00;

	// Timer1 Off
	TCCR1A=0x00;
	TCCR1B=0x00;
	TCNT1H=0x00;
	TCNT1L=0x00;
	ICR1H=0x00;
	ICR1L=0x00;
	OCR1AH=0x00;
	OCR1AL=0x00;
	OCR1BH=0x00;
	OCR1BL=0x00;

	// PWM's auf Low (0)
	REDPORT &= ~(1 << REDPINNR);
	GREENPORT &= ~(1 << GREENPINNR);
	BLUEPORT &= ~(1 << BLUEPINNR);
	WHITEPORT &= ~(1 << WHITEPINNR);
	
	// PWM's als Ausgaenge (1)
	REDDDR |= (1 << REDPINNR);
	GREENDDR |= (1 << GREENPINNR);
	BLUEDDR |= (1 << BLUEPINNR);	
	WHITEDDR |= (1 << WHITEPINNR);
	
	// Stromsparmodus wieder einschalten
	PowerSave();
}


/**
* \brief
* Initialisiert die Timer und die Ausgaenge fuer PWM
*/
void IntiPWMPeripherie ( void )
{
	power_timer0_enable();
	power_timer1_enable();

	Init_PWMPins();
	Init_PWMTimers();
	Init_PWMInterrupt();
}


/**
* \brief
* Schaltet den Piepser aus
*/
void BuzzerOff ( void )
{
	// Timer/Counter 2 off
	ASSR=0x00;
	TCCR2A=0x00;
	TCCR2B=0x00;
	TCNT2=0x00;
	OCR2A=0x00;
	OCR2B=0x00;

	// Timer2 Overflow Interrupt deaktivieren
	TIMSK2 &= ~(1 << TOIE2);

	// Buzzer ausschalten
	BUZZERPORT &= ~(1 << BUZZERPINNR);
}


/**
* \brief
* Schaltet den Piepser an
*/
void BuzzerOn ( void )
{
	power_timer2_enable();

	// Timer/Counter 2 initialization
	// Clock source: System Clock
	// Clock value: 3.906 kHz
	// Mode: Normal top=FFh
	// OC2A output: Disconnected
	// OC2B output: Disconnected
	ASSR=0x00;
	TCCR2A=0x00;
	TCCR2B=0x06;
	TCNT2=0x00;
	OCR2A=0x00;
	OCR2B=0x00;

	// Timer2 Overflow Interrupt
	TIMSK2 |= (1 << TOIE2);
}
