/******
 ADC.c
******/


//***********
//* Includes
//***********
#include "adc.h"


/**
* \brief
* Liest einen Analogeingang ein. Die Referenz und der Kanal und  muss vorher gewaehlt werden
* \note 
* Die Referenz muss vor dem Kanal gesetzt werden
* \return
* 10bit AD-Wert
*/
unsigned int ADC_Read ( void )
{
	int Resultat;
	int i;

	// Prescaler auf 16
	// ADC-Frequenz = 1MHz / 16 = 62.5 kHz
	ADCSRA |= (1 << ADPS2);

	// ADC Enablen
	ADCSRA |= (1 << ADEN);

	// Dummy Read
	ADCSRA |= (1<<ADSC);
	while (ADCSRA & (1<<ADSC)) 
	{
		;
	}

	// "single conversion" einstellen
    ADCSRA |= (1<<ADSC);							
	Resultat = 0;

	// Eigentliche Messung
	for (i=0; i<4; i++)
	{
		// auf Abschluss der Konvertierung warten
		while (ADCSRA & (1<<ADSC)) {;}

		// Messung aufaddieren
		Resultat += ADCW;
	}								 

 	// Resultat durch vier teilen
	Resultat /= 4;

	// ADC Disable
	ADCSRA &= ~(1 << ADEN);
 
  return Resultat;
}


/**
* \brief
* Setzt die Kanalnummer
* \param
* mux Kanal 0 bis 7
* \note
* Vorher muss die Referenz festgelegt werden
*/
void Set_Channel ( unsigned char mux )
{
	unsigned char byte;

	byte = ADMUX;
	byte = (byte & 0xF0);
	byte = (byte | mux);

	ADMUX = byte;
}


/**
* \brief
* Waehlt die interne Referenzspannung aus
* \note
* 1.1V
*/
void Set_ADRefInternal ( void )
{
	ADMUX |= ((1<<REFS1) | (1<<REFS0));
}


/**
* \brief
* Liest die Spannungen ein
* \note
* Die Spannungen werden in den lokalen Variabeln: Voltage_3V, Voltage_6V gespeichert. Einheit: mV
*/
void Read_Voltages ( void )
{   
	unsigned int ADWert;

	// AD-Wandler einschalten
	power_adc_enable();
	_delay_ms(100);

	/*-------------------------------
	 Spannung = ReferenzSpannung /
	 Aufloesung * ADC

	 U = 1100mV / 1024 * ADWert * Spannungsteiler
	 U = 1.074 * ADWert * 4.39
	-------------------------------*/

		// 1.1V als Referenz
		Set_ADRefInternal();
		_delay_ms(1);

		// Spannung messen
		Set_Channel(UBATT_PINNR);	
		ADWert = ADC_Read();
		
		// Spannung ausrechnen
		ADWert *= 22;
		ADWert /= 5;
		Voltage_Batt = ADWert; 
	
	/*-------------------------------
	 Spannung = ReferenzSpannung /
	 Aufloesung * ADC

	 U = 1100mV / 1024 * ADWert * Spannungsteiler
	 U = 1.074 * ADWert * 11.8
	-------------------------------*/
		
		// 1.1V als Referenz
		Set_ADRefInternal();
		_delay_ms(1);

		// Spannung messen
		Set_Channel(UAKKU_PINNR);	
		ADWert = ADC_Read();
		
		// Spannung ausrechnen
		ADWert *= 12;
		ADWert -= 80;
		Voltage_Accu = ADWert; 

	// AD-Wandler ausschalten
	power_adc_disable();
}
