/**************
 application.c
***************/


//***********
//* Includes
//***********
#include "application.h"


//***********
//* Variabeln
//***********
const char WeekdayTable[7][2] = {"So","Mo","Di","Mi","Do","Fr","Sa"};


/**
* \brief
* Zaehlt die Zeit um eine Sekunde rauf
*/
void Inc_Time ( void )
{
	Zeit.SecChanged = TRUE;
	RTC.Sec++;

	if (RTC.Sec > 59)
	{
		RTC.Sec -= 60;
		RTC.Min++;
		Zeit.MinChanged = TRUE;
		TaskHandler_AddTask(CheckAlarm);
	}

	if (RTC.Min > 59)
	{
		RTC.Min -= 60;
		RTC.Hour++;
	}

	if (RTC.Hour > 23)
	{
		PCF8563_GetTime();
	}

	Zeit.LastSec = RTC.Sec;
	Zeit.LastMin = RTC.Min;
}


/**
* \brief
* Stellt eine Zeit ein, diese wird laufend auf Zeile 2 angezeigt
* \param
* *Stunden Pointer auf die Variable, in der die Stunden stehen und gespeichert werden
* \param
* *Minuten Pointer auf die Variable, in der die Minuten stehen und gespeichert werden
* \param
* *BusyFlag Pointer auf die Variable, in der das BusyFlag gespeichert ist
*/
void SetTime ( unsigned char *Stunden, unsigned char *Minuten, unsigned char *BusyFlag )
{
	static unsigned char Ziffer;
	static unsigned char h10;
	static unsigned char h1;
	static unsigned char m10;
	static unsigned char m1;
	static unsigned char h;
	static unsigned char m;

	if (*BusyFlag == FALSE)
	{
		h = *Stunden;
		m = *Minuten;

		SplitNumber(h, &Ziffer, &Ziffer, &h10, &h1);
		SplitNumber(m, &Ziffer, &Ziffer, &m10, &m1);

		Ziffer = 0;
		*BusyFlag = TRUE;
		Zeit.MinChanged = TRUE;
	}

	if (OK.Kurz == TRUE)
	{
		OKEntprellen();
		OK.Kurz = FALSE;
		Ziffer++;
		Zeit.MinChanged = TRUE;
	}
	
	if (ESC.Kurz == TRUE)
	{
		ESCEntprellen();
		ESC.Kurz = FALSE;
		*BusyFlag = FALSE;
		Disp_Cursor_Off();
		AppLayer--;
		return;
	}

	if (Next.Kurz == TRUE)
	{
		NextEntprellen();
		Next.Kurz = FALSE;
					
		switch (Ziffer)
		{
			case 0:
				IncChar(&h10, 2, 0);
				break;

			case 1:
				if (h10 < 2)
				{
					IncChar(&h1, 9, 0);
				}
				else
				{
					IncChar(&h1, 3, 0);
				}
				break;

			case 2:
				IncChar(&m10, 5, 0);
				break;

			case 3:
				IncChar(&m1, 9, 0);
				break;
		}
		
		Zeit.MinChanged = TRUE;
	}
	
	if (Ziffer == 4)
	{
		// Fertig eingestellt
		*BusyFlag = FALSE;
		Disp_Cursor_Off();
		*Stunden = h;
		*Minuten = m;
		AppLayer--;
	}

	// Zeit anzeigen
	if ((Zeit.MinChanged == TRUE) && (Ziffer < 4))
	{
		h = h10 * 10 + h1;
		m = m10 * 10 + m1;
		Disp_Clear_Line(1);
		Disp_SetAddr(1, 0);
		Disp_Integer(h10);
		Disp_Integer(h1);
		Disp_Char(':');
		Disp_Integer(m10);
		Disp_Integer(m1);

		if (Ziffer < 2)
		{
			Disp_SetAddr(1, Ziffer);
		}
		else
		{
			Disp_SetAddr(1, Ziffer + 1);
		}
		Disp_Cursor_On();
	}
}


/**
* \brief
* Stellt das Datum ein, dieses wird laufend auf Zeile 2 angezeigt
* \param *Day Pointer auf die Variable, in der die Tage stehen und gespeichert werden
* \param *Month Pointer auf die Variable, in der die Monate stehen und gespeichert werden
* \param *Year Pointer auf die Variable, in der die Jahre stehen und gespeichert werden
* \param *Weekday Pointer auf die Variable, in der der Wochentag stehen und gespeichert werden
* \param *BusyFlag Pointer auf die Variable, in der das BusyFlag gespeichert ist
*/
void SetDate ( unsigned char *Year, unsigned char *Month, unsigned char *Day, unsigned char *Weekday, unsigned char *BusyFlag )
{
	static unsigned char Ziffer;
	static unsigned char d10;
	static unsigned char d1;
	static unsigned char m10;
	static unsigned char m1;
	static unsigned char y10;
	static unsigned char y1;
	static unsigned char d;
	static unsigned char m;
	static unsigned char y;
	static unsigned char w;

	if (SetDateBusy == FALSE)
	{
		d = *Day;
		m = *Month;
		y = *Year;
		w = *Weekday;

		SplitNumber(d, &Ziffer, &Ziffer, &d10, &d1);
		SplitNumber(m, &Ziffer, &Ziffer, &m10, &m1);
		SplitNumber(y, &Ziffer, &Ziffer, &y10, &y1);

		Ziffer = 0;
		*BusyFlag = TRUE;
		Zeit.MinChanged = TRUE;
	}

	if (OK.Kurz == TRUE)
	{
		OKEntprellen();
		OK.Kurz = FALSE;
		Ziffer++;
		Zeit.MinChanged = TRUE;
	}
	
	if (ESC.Kurz == TRUE)
	{
		ESCEntprellen();
		ESC.Kurz = FALSE;
		SetDateBusy = FALSE;
		Disp_Cursor_Off();
		AppLayer--;
		return;
	}

	if (Next.Kurz == TRUE)
	{
		NextEntprellen();
		Next.Kurz = FALSE;

		switch (Ziffer)
		{
			case 0:
				IncChar(&d10, 3, 0);
				break;

			case 1:
				IncChar(&d1, 9, 0);
				break;

			case 2:
				IncChar(&m10, 1, 0);
				break;

			case 3:
				IncChar(&m1, 9, 0);
				break;

			case 4:
				IncChar(&y10, 9, 0);
				break;

			case 5:
				IncChar(&y1, 9, 0);
				break;

			case 6:
				IncChar(&w, 6, 0);
				break;
		}
		Zeit.MinChanged = TRUE;
	}
	
	if (Ziffer == 7)
	{
		// Fertig eingestellt
		Disp_Cursor_Off();
		*BusyFlag = FALSE;
		*Weekday = w;
		*Day = d;
		*Month = m;
		*Year = y;
		AppLayer--;
	}

	// Zeit anzeigen
	if ((Zeit.MinChanged == TRUE) && (Ziffer < 7))
	{
		d = d10 * 10 + d1;
		m = m10 * 10 + m1;
		y = y10 * 10 + y1;
		Disp_Clear_Line(1);
		Disp_SetAddr(1, 0);
		Disp_Integer(d10);
		Disp_Integer(d1);
		Disp_Char('.');
		Disp_Integer(m10);
		Disp_Integer(m1);
		Disp_String(".20");
		Disp_Integer(y10);
		Disp_Integer(y1);
		Disp_Char(' ');
		Disp_Char(WeekdayTable[w][0]);
		Disp_Char(WeekdayTable[w][1]);

		if (Ziffer < 2)
		{
			Disp_SetAddr(1, Ziffer);
		}
		else if (Ziffer < 4)
		{
			Disp_SetAddr(1, Ziffer + 1);
		}
		else if (Ziffer < 6)
		{
			Disp_SetAddr(1, Ziffer + 4);
		}
		else
		{
			Disp_SetAddr(1, Ziffer + 6);
		}
		Disp_Cursor_On();
	}
}


/**
* \brief
* Stellt ein, an welchen Tagen der Alarm eingeschaltet ist
* \param
* *BusyFlag Pointer auf die Variable, in der das BusyFlag gespeichert ist
*/
void SetAlarmWeekdays ( unsigned char *BusyFlag )
{
	static unsigned char AlarmDays;
	static unsigned char Ziffer;
	unsigned char i;

	if (*BusyFlag == FALSE)
	{
		AlarmDays = Alarm.Weekdays;
		
		if (AlarmDays & 0x01)
		{
			AlarmDays |= 0x80;
		}
		AlarmDays >>= 1;
		
		Ziffer = 0;
		*BusyFlag = TRUE;
		Zeit.MinChanged = TRUE;
	}

	if (OK.Kurz == TRUE)
	{
		OKEntprellen();
		OK.Kurz = FALSE;
		Ziffer++;
		Zeit.MinChanged = TRUE;
	}
	
	if (ESC.Kurz == TRUE)
	{
		ESCEntprellen();
		ESC.Kurz = FALSE;
		*BusyFlag = FALSE;
		Zeit.MinChanged = TRUE;
		return;
	}

	if (Next.Kurz == TRUE)
	{
		NextEntprellen();
		Next.Kurz = FALSE;
		
		Zeit.MinChanged = TRUE;
		AlarmDays ^= (1 << Ziffer); 
	}
	
	if (Zeit.MinChanged == TRUE)
	{
		Disp_SetAddr(1, 0);
		Disp_String("Mo");
		
		for (i=0; i<7; i++)
		{
			if (AlarmDays & (1 << i))
			{
				// Alarm ein
				Disp_Char(SQUARE);
			}
			else
			{
				// Alarm aus
				Disp_Char(UNDERLINE);
			}
		}
		
		Disp_String("So");
		Disp_SetAddr(1, (Ziffer + 2));
		Disp_Cursor_On();
	}
	
	if (Ziffer == 7)
	{
		// Fertig eingestellt
		*BusyFlag = FALSE;
		Zeit.MinChanged = TRUE;
		Disp_Cursor_Off();
		AlarmDays <<= 1;
		if (AlarmDays & 0x80)
		{
			AlarmDays++;
		}
		Alarm.Weekdays = AlarmDays;
		
		WriteAlarmState();
	}
}


/**
* \brief
* Schreibt die Standard Anzeige auf das LCD
* \note
* Zeile1: Al:xxx
* Zeile2: dd.mm.yy hh.mm A
*/
void LCD_StandardPrompt ( void )
{
	unsigned char Weekday;
	
	Disp_Clear();

	// Erste Zeile
	if (Voltage_Accu < ACCUVOLTAGEMIN)
	{
		Disp_SetAddr(0, 0);
		Disp_String("Low");
	}
	else
	{
		// Wochentag aus Platzgruenden nur anzeigen, wenn nich LowBatt
		Disp_SetAddr(0, 0);
		Disp_Char(WeekdayTable[RTC.Weekday][0]);
		Disp_Char(WeekdayTable[RTC.Weekday][1]);
	}
	
	if (Alarm.State == TRUE)
	{
		if (TimeDiff(&RTC.Hour, &RTC.Min, &Alarm.Hour, &Alarm.Min) < 0xFFFF)
		{
			// Alarm noch heute
			Weekday = (1 << RTC.Weekday);
		}
		else
		{
			// Alarm erst morgen wieder
			Weekday = (1 << (RTC.Weekday + 1));
			if (Weekday == 0x80)
			{
				Weekday = 0x01;
			}
		}
		
		if (Alarm.Weekdays & Weekday)
		{
			Disp_SetAddr(0, 6);
			Disp_String("Al");
			Disp_SetAddr(0, 9);
			LCD_WriteTime(&Alarm.Hour, &Alarm.Min);
		}
		else
		{
			Disp_SetAddr(0, 11);
			Disp_String("Al");
			Disp_SetAddr(0, 14);
			Disp_String("ON");
		}
	}
	else
	{
		Disp_SetAddr(0, 10);
		Disp_String("Al");
		Disp_SetAddr(0, 13);
		Disp_String("OFF");
	}

	// Zweite Zeile
	Disp_SetAddr(1, 0);
	Disp_Integer00(RTC.Day);
	Disp_Char('.');
	Disp_Integer00(RTC.Month);
	Disp_Char('.');
	Disp_Integer00(RTC.Year);

	Disp_SetAddr(1, 9);
	LCD_WriteTime(&RTC.Hour, &RTC.Min);
}


/**
* \brief
* Schreibt eine Zeit im AM/PM Format auf das LCD
* \note
* Die Position muss verher festgelegt werden
* \param
* *Hour Pointer auf die Stundenvariable (0-23)
* \param
* *Min Pointer auf die Minutenvariabel (0-59)
*/
void LCD_WriteTime (unsigned char *Hour, unsigned char *Min)
{
	if (*Hour >= 12)
	{
		// PM
		if (*Hour > 12)
		{
			Disp_Integer00(*Hour - 12);
		}
		else
		{
			Disp_Integer00(*Hour);
		}

		Disp_Char(':');
		Disp_Integer00(*Min);
		Disp_String(" P");
	}
	else
	{
		// AM
		Disp_Integer00(*Hour);
		Disp_Char(':');
		Disp_Integer00(*Min);
		Disp_String(" A");
	}
}


/**
* \brief
* Ueberprueft, ob das Wake-Up-Light eingeschaltet werden soll
* \note
* Darf nur jede Minute aufgerufen werden
*/
void CheckAlarm ( void )
{
	unsigned int ZeitDifferenz;

	if (Alarm.State == TRUE)
	{
		if (Alarm.Weekdays & (1 << RTC.Weekday))
		{
			ZeitDifferenz = TimeDiff (&RTC.Hour, &RTC.Min, &Alarm.Hour, &Alarm.Min);
		
			if (ZeitDifferenz < (LIGHT_LEED_TIME + 1))
			{
				if (Alarm.On == FALSE)
				{
					// verhindert, dass mehrmals initialisiert wird
					IntiPWMPeripherie();
					Alarm.On = TRUE;
				}
			}

			if (ZeitDifferenz == 0)
			{
				// Timer0 Overflow Interrupt deaktivieren
				TIMSK0 &= ~(1 << TOIE0);

				// PWM Ausgaenge
				REDPWM = 64;
				GREENPWM = 64;
				BLUEPWM = 64;
				WHITEPWM = 255;

				// Buzzer einschalten
				BuzzerOn();

				// Weisse LED einschalten
				TaskScheduler_RemoveTask(QuitAlarm);
				TaskScheduler_AddTask(QuitAlarm, BUZZER_TIME_ON);
			}
		}
	}
	else
	{
		if (Alarm.On == TRUE)
		{
			// Licht ist noch an und muss ausgeschaltet werden
			QuitAlarm();
		}
	}
}


/**
* \brief
* Wechselt in den Sleep-Mode
*/
void EnterSleep ( void )
{
	set_sleep_mode(SLEEP_MODE_PWR_DOWN);
	sleep_enable();
	sleep_cpu();
	sleep_disable();
}


/**
* \brief
* Beendet den Alarm Modus und wechselt dann in den Sleepmodus
*/
void QuitAlarm ( void )
{
	// PWM Ausgaenge abschalten
	UnIntiPWMPeripherie();

	// Buzzer ausschalten
	BuzzerOff();

	// Sllep wieder enable
	Alarm.On = FALSE;
	
	// Stromsparmodus wieder einschalten
	PowerSave();
	
}


/**
* \brief
* Schaltet alle LEDs ein
*/
void LEDsOn ( void )
{
	REDPORT |= (1 << REDPINNR);
	GREENPORT |= (1 << GREENPINNR);
	BLUEPORT |= (1 << BLUEPINNR);
	WHITEPORT |= (1 << WHITEPINNR);
}


/**
* \brief
* Schaltet alle LEDs aus
*/
void LEDsOff ( void )
{
	REDPORT &= ~(1 << REDPINNR);
	GREENPORT &= ~(1 << GREENPINNR);
	BLUEPORT &= ~(1 << BLUEPINNR);
	WHITEPORT &= ~(1 << WHITEPINNR);
}


/**
* \brief
* Speichert die Alarmeinstelungen im RTC ab
*/
void WriteAlarmState ( void )
{
	unsigned char Weekday;
	
	// Stunden und Minuten schreiben
	PCF8563_WriteRegister(ADDR_ALARM_HOURS, BinaryToBCD(Alarm.Hour));
	PCF8563_WriteRegister(ADDR_ALARM_MINUTE, BinaryToBCD(Alarm.Min));
	
	// Wenn Alarm eingeschaltet wird Bit 7 von Weekdays gesetzt
	Weekday = Alarm.Weekdays;
	if (Alarm.State == TRUE)
	{
		Weekday |= 0x80;
	}
	
	// Weekday und Ein/Aus schreiben
	// Bit 0 bis 5
	PCF8563_WriteRegister(ADDR_ALARM_DAY, (Weekday & 0x3f));
	// Bit 6 und 7
	Weekday &= 0xc0;
	Weekday >>= 6;
	PCF8563_WriteRegister(ADDR_ALARM_WEEKDAY, Weekday);
}


/**
* \brief
* Liest die Alarmeinstellungen aus dem RTC
*/
void ReadAlarmState ( void )
{
	unsigned char Weekday;
	
	// Stunden und Minuten lesen und speichern
	Alarm.Hour = BCDToBinary(PCF8563_ReadRegister(ADDR_ALARM_HOURS) & MASK_HOURS);
	if (Alarm.Hour > 23)
	{
		Alarm.Hour = 23;
	}
	Alarm.Min = BCDToBinary(PCF8563_ReadRegister(ADDR_ALARM_MINUTE) & MASK_MINUTES);
	if (Alarm.Min > 59)
	{
		Alarm.Min = 59;
	}
	
	// Weekday und Ein/Aus lesen
	// Bit 6 und 7
	Weekday = PCF8563_ReadRegister(ADDR_ALARM_WEEKDAY);
	Weekday &= 0x03;
	Weekday <<= 6;
	// Bit 0 bis 5
	Weekday |= (PCF8563_ReadRegister(ADDR_ALARM_DAY) & 0x3f);
	
	// Ein/Aus speichern
	if (Weekday & 0x80)
	{
		Alarm.State = TRUE;
	}
	else
	{
		Alarm.State = FALSE;
	}
	
	// Weekday speichern
	Alarm.Weekdays = (Weekday & 0x7f);
}


/**
* \brief
* Macht einen Softwarereset
*/
void Reset ( void )
{
	wdt_enable(WDTO_15MS);
	while(1)
	{
		;
	}
}


/*void THOV ( void )
{
	cli();
	Disp_Clear();
	Disp_SetAddr(0, 0);
	Disp_String("Task Handler");
	while(1) {;}
}


void TSOV ( void )
{
	cli();
	Disp_Clear();
	Disp_SetAddr(0, 0);
	Disp_String("Scheduler Handler");
	while(1) {;}
}*/
