/**
* \brief
* Displays mit ST7036 Controller ueber SPI ansteuern
* \version
* 1.2 Kontrast kann Spannungsabhaengig eingestellt werden <br>
* 1.1 Cursor Funktion hinzugefuegt <br>
* 1.0 Von Christopher Mueller uebernommen und angepasst
* \date
* Create : 17. Oktober 2007, Christopher Mueller <br>
* Modify : 11. Januar 2009, Philipp Kaelin
* \author
* Original : Christopher Mueller  <A HREF="http://www.shortysfastlane.de"> www.shortysfastlane.de </A> <br>
* Erweitert und verbessert: Philipp Kaelin
*/

#ifndef _DOGM_H_
	#define _DOGM_H_

	//***********
	//* Includes
	//***********
	#include <util/delay.h>
	#include <avr/io.h>
	#include <stdlib.h>
	#include "adc.h"
	#include "scheduler.h"


	//**********
	//* Defines
	//**********
	// CLK
	#define DDR_CLK			DDRB
	#define PORT_CLK		PORTB
	#define PINNR_CLK		PORTB5

	// SI
	#define DDR_SI			DDRB
	#define PORT_SI			PORTB
	#define PINNR_SI		PORTB3
	
	// RS
	#define DDR_RS			DDRB
	#define PORT_RS			PORTB
	#define PINNR_RS		PORTB0
	
	// CSB
	#define DDR_CSB			DDRD
	#define PORT_CSB		PORTD
	#define PINNR_CSB		PORTD7 
	
	// Backlight
	#define DDR_BACKLIGHT	DDRD
	#define PORT_BACKLIGHT	PORTD
	#define PINNR_BACKLIGHT	PORTD1


	//*************
	//* Funktionen
	//*************
	
	// Software Funktionen
	// --------------------------
	void Disp_Clear ( void );
	
	void Disp_Clear_Line ( unsigned char ucLine );
	
	void Disp_Char ( char cData );
	
	void Disp_String ( char *cData );
	
	void Disp_Integer ( int iData );

	void Disp_Integer00 ( int iData );
	
	void Disp_Command ( unsigned char Data );
	
	void Disp_Byte ( unsigned char Data );
	
	void Disp_SetAddr ( unsigned char ucLine, unsigned char ucRow );

	void Disp_Cursor_On ( void );

	void Disp_Cursor_Off ( void );


	// Hardware Funktionen
	// --------------------------
	void Disp_Init ( unsigned char Contrast );

	void Disp_InitHW ( void );

	void Disp_AdjustContrast ( void );
	
	void SPI_SendByte ( unsigned char ucData );

	void SetBacklightOn ( void );

	void SetBacklightOff ( void );
	
#endif
