/*********
 global.c
**********/


//***********
//* Includes
//***********
#include "global.h"


//************
//* Variabeln
//************
tTIMEBASE 		Zeit 		= {0, TRUE, TRUE, 0, 0};
tAUSWAHL		Auswahl 	= {0, 0, 0, 0, 1, TRUE, FALSE, FALSE};
tTASTER			Next 		= {FALSE, FALSE, 0};
tTASTER 		OK 			= {FALSE, FALSE, 0};
tTASTER 		ESC 		= {FALSE, FALSE, 0};
tTASTER 		Light 		= {FALSE, FALSE, 0};
tALARM			Alarm 		= {FALSE, 0, 0, FALSE};


unsigned char AppMode[MODE_DEPTH];
unsigned char AppLayer = 0;
volatile unsigned char ApplicationBusy = FALSE;
unsigned char SetTimeBusy = FALSE;
unsigned char SetDateBusy = FALSE;
unsigned char SetAlarmBusy = FALSE;
unsigned char SetAlarmDaysBusy = FALSE;
unsigned char FunctionBusy = FALSE;
unsigned char PWMLightOn = FALSE;
unsigned int Voltage_Batt = 0;
unsigned int Voltage_Accu = 0;

volatile unsigned int ColorCnt;
unsigned char RedTableCnt;
unsigned char GreenTableCnt;
unsigned char BlueTableCnt;


/**
* \brief
* Rechnet den Absolutwert von zwei vorzeichenlose Integer Zahlen aus
* \param 
* value1 (0 - 32767) 
* \param
* value2 (0 - 32767)
* \return
* Absolutwert als Integer 
*/
unsigned int abs_uint(unsigned int value1, unsigned int value2)
{
	unsigned int result;

	if (value1 > value2)
	{
		result = value1 - value2;
	}
	else
	{
		result = value2 - value1;
	}

	return result;
}


/**
* \brief
* Gibt das HighByte aus einem 16bit Datentyp zurueck
* \param Data (16bit Datentyp)
* \return HighByte des 16bit Datentyps)
*/
unsigned char GetHiByte(unsigned int Data)
{
	unsigned int Temp;

	// HighByte herauskopieren
	Temp = Data >> 8;
	return (unsigned char) Temp;
}


/**
* \brief
* Gibt das LowByte aus einem 16bit Datentyp zurueck
* \param Data (16bit Datentyp)
* \return (LowByte des 16bit Datentyps)
*/
unsigned char GetLoByte(unsigned int Data)
{
	unsigned int Temp;

	// LowByte herauskopieren
	Temp = Data & 0xFF;
	return (unsigned char) Temp;
}


/**
* \brief
* Inkrementiert eine Variable unter Beachtung des Maximalwertes
* \param
* *Var Pointer auf die Variable, die inkrementiert werden soll
* \param
* UpperBound Oberer Grenzwert der Variable
* \param
* ResetValue Wert auf welchen die Variable zurueckgesetzt werden soll
*/
void IncChar ( unsigned char *Var, unsigned char UpperBound, unsigned char ResetValue )
{
	(*Var)++;

	if (*Var > UpperBound)
	{
		*Var = ResetValue;
	}
}


/**
* \brief
* Dekrementiert eine Variable unter Beachtung des Minimalwertes
* \param
* *Var Pointer auf die Variable, die inkrementiert werden soll
* \param
* LowerBound Oberer Grenzwert der Variable
* \param
* ResetValue Wert auf welchen die Variable zurueckgesetzt werden soll
*/
void DecChar ( unsigned char *Var, unsigned char LowerBound, unsigned char ResetValue )
{
	if (*Var > LowerBound)
	{
		(*Var)--;
	}
	else
	{
		*Var = ResetValue;
	}
}

/**
* \brief
* Teilt eine Zahl in Tausender, Hunderter, Zehner und Einer auf
* \param
* Zahl Zahl, die aufgeteilt werden soll
* \param
* *Tausender Pointer auf die Variable der Tausender
* \param
* *Hunderter Pointer auf die Variable der Hunderter
* \param
* *Zehner Pointer auf die Variable der Zehner
* \param
* *Einer Pointer auf die Variable der Einer
*/
void SplitNumber ( unsigned int Zahl, unsigned char *Tausender, unsigned char *Hunderter, unsigned char *Zehner, unsigned char *Einer )
{
	unsigned char z = Zahl;

	*Tausender = 0;
	*Hunderter = 0;
	*Zehner = 0;
	*Einer = 0;

	while (z > 999)
	{
		z -= 1000;
		*Tausender = *Tausender + 1;
	}

	while (z > 99)
	{
		z -= 100;
		*Hunderter = *Hunderter + 1;
	}

	while (z > 9)
	{
		z -= 10;
		*Zehner = *Zehner + 1;
	}

	*Einer = z;
}


/**
* \brief
* Berechnet die Differenz von zwei Zeiten
* \note
* Zeit = Zeit2 - Zeit1 <br>
* Wenn die Zeit1 groesser als die Zeit2 ist, wird als differenz 0xFFFF zurueckgegeben
* \param
* *Hour1 Pointer auf die kleinere Zeit
* \param
* *Min1 Pointer auf die kleinere Zeit
* \param
* *Hour2 Pointer auf die groessere Zeit
* \param
* *Min2 Pointer auf die groessere Zeit
*/
unsigned int TimeDiff ( unsigned char *Hour1, unsigned char *Min1, unsigned char *Hour2, unsigned char *Min2 )
{
	unsigned int time1;
	unsigned int time2;
	
	time1 = *Hour1;
	time1 *= 60;
	time1 += *Min1;
	
	time2 = *Hour2;
	time2 *= 60;
	time2 += *Min2;
	
	if (time2 < time1)
	{
		return 0xFFFF;
	}
	
	time2 -= time1;
	return time2;
}

