/**
* \file
* global.h
* \brief
* In dieser Datei sind Variablen, Funktionen und Defines die im ganzen Programm zur verfuegugn stehen muessen
* \author
* Philipp Kaelin 
*/

#ifndef _GLOBAL_H_
	#define _GLOBAL_H_


	//***********
	//* Includes
	//***********
	#include <avr/pgmspace.h>
	

	//**********
	//* Defines
	//**********
	//#define DEBUG_ENABLED

	// Allgemein
	#define TRUE					1
	#define FALSE					0
	#define STDCONTRAST				0x78
	#define ACCUVOLTAGEMIN			5200 // 2x 2,6V
	
	// Special LCD Chars
	#define UNDERLINE				0x5f
	#define SQUARE					0xdb
	
	// Tasten
	#define TASTELANG				3 - 1

	// PinChangeInterrupt
	#define PCI_CLK 				(PIND & (1 << PIND2))
	#define PCI_SWMODE				(PIND & (1 << PIND4))
	#define PCI_SWNEXT				(PIND & (1 << PIND0))

	// Zeiten
	#define BACKLIGHT_TIME_ON		5		// sec
	#define BUZZER_TIME_ON			20		// sec
	#define LIGHT_LEED_TIME			15		// min
	#define LIGHT_LAG_TIME			60		// sec

	// Modis 
	#define MODE_DEPTH				3
	
	// Layer 0
	#define MODE_ENTRY_MAX			2
	#define MODE_STDSCREEN			0
	#define MODE_ALARM				1
	#define MODE_SETTINGS			2

	// Layer 1 (Alarm)
	#define ALARM_ENTRY_MAX			2
	#define ALARM_ONOFF				0
	#define ALARM_SET				1
	#define ALARM_DAYS				2
	
	// Layer 1 (Einstellungen)
	#define SETTINGS_ENTRY_MAX		2
	#define SETTINGS_TIME			0
	#define SETTINGS_DATE			1
	#define SETTINGS_VOLTAGES		2
	
	// Layer 2 (Alarm)
	#define SETALARM_ONOFF_MAX		1
	#define SETALARM_ON				0
	#define SETALARM_OFF			1	
	
	
	//************
	//* Variabeln
	//************
	/** TRUE wenn die Applikationsfunktion gerade ausgefhrt wird */
	extern volatile unsigned char ApplicationBusy;
	
	/** TRUE wenn gerade eine Zeit eingestellt wird */
	extern unsigned char SetTimeBusy;

	/** TRUE wenn gerade das Datum eingestellt wird */
	extern unsigned char SetDateBusy;
	
	/** TRUE wenn gerade einen Alarmzeit eingestellt wird */
	extern unsigned char SetAlarmBusy;
	
	/** TRUE wenn gerade die Alarmtage eingestellt werden */
	extern unsigned char SetAlarmDaysBusy;
	
	/** TRUE wenn vom Menue eine Funktion aufgerufen wurde */
	extern unsigned char FunctionBusy;
	
	/** TRUE wenn das PWM Licht eingeschaltet ist */
	extern unsigned char PWMLightOn;

	/** Spannung der Batterie in mV */
	extern unsigned int Voltage_Batt;

	/** Spannung des Akkus in mV */
	extern unsigned int Voltage_Accu;

	/** Definiert den Modus, in dem die Anwendung jetzt gerade ist */
	extern unsigned char AppMode[MODE_DEPTH];
	
	/** Speichert in welchem Menue-Layer man sich gerade befindet */
	extern unsigned char AppLayer;

	extern volatile unsigned int ColorCnt;

	extern unsigned char RedTableCnt;
	extern unsigned char GreenTableCnt;
	extern unsigned char BlueTableCnt;
	
	
	/** Wird alle Sekunde aktualisiert */
	typedef struct 
	{
		/** Sekundenzaehler */
		unsigned int	totSekunden;
		/** TRUE sobald die Sekunden geaendert hat  */
		unsigned char	SecChanged;
		/** TREU sobald die Minuten geaendert hat */
		unsigned char	MinChanged;
		/** Letzte Sekunde wird hier gespeichert um zu erkennen ob sich die Sekunde geaendert hat */
		unsigned char	LastSec;
		/** Letzte Minute wird hier gespeichert um zu erkennen ob sich die Minuten geaendert haben */
		unsigned char	LastMin;
	} tTIMEBASE;

	/** Wird aktualisiert sobald der Encoder gedreht wird und Enable auf TRUE ist */
	typedef struct
	{
		/** Maximalwert von Counter */
	    unsigned int   MaxEintraege;
		/** Minimalwert von Counter */
	    unsigned char	MinEintraege;
		/**  Counter */
	    signed	long	Count;
		/** Schrittaenderung bei normalem drehen */
		unsigned char	Step;
		/** Schrittaenderung bei schnellem drehen */
		unsigned int	BigStep;
		/** Wenn TRUE wird bei einem Counter ueberlauf am anderen Ende weitergezaehlt */
		unsigned char	Roll;
		/** TRUE wenn Counter geaendert hat */
		unsigned char	Changed;
		/** TRUE falls Aenderungen am Encoder ausgewertet werden sollen */
		unsigned char	Enable;
	} tAUSWAHL;	
	
	/** Tastereinstellungen */
	typedef struct
	{
		/** TRUE wenn Taste kurz gedrueckt */
		unsigned char Kurz;
		/** TRUE wenn Taste lange gedrueckt */
		unsigned char Lang;
		/** Zaehl die Sekunden seit die Taste gedrueckt ist */
		unsigned char Counter;
	} tTASTER;

	/** Einstellungen zum Alarm	*/
	typedef struct
	{
		/** Bit0 Sonntag, Bit6 Samsttag */
		unsigned char Weekdays;
		/** TRUE wenn Eingeschltet, ansonsten FALSE */
		unsigned char State;
		/** Alarm Stunde */
		unsigned char Hour;
		/** Alarm Minuten */
		unsigned char Min;
		/** TRUE wenn Licht gerade an */
		unsigned char On;
	} tALARM;

	extern tTIMEBASE 		Zeit;
	extern tAUSWAHL			Auswahl;
	extern tTASTER 			Next, ESC, OK, Light;
	extern tALARM 			Alarm;
	
	
	
	//*************************
	//* Funktionsdeklarationen
	//*************************

	unsigned int abs_uint(unsigned int value1, unsigned int value2);

	unsigned char GetHiByte(unsigned int Data);

	unsigned char GetLoByte(unsigned int Data);

	void IncChar ( unsigned char *Var, unsigned char UpperBound, unsigned char ResetValue );
	
	void DecChar ( unsigned char *Var, unsigned char LowerBound, unsigned char ResetValue );

	void SplitNumber ( unsigned int Zahl, unsigned char *Tausender, unsigned char *Hunderter, unsigned char *Zehner, unsigned char *Einer );
	
	unsigned int TimeDiff ( unsigned char *Hour1, unsigned char *Min1, unsigned char *Hour2, unsigned char *Min2 );

#endif
