/*******
 init.c
********/


//***********
//* Includes
//***********
#include "init.h"

/**
* \brief
* Initialisiert die Hardwarekonfiguration des RTC
*/
void Init_RTCPins ( void )
{
	// PCI17 als Eingang mit PullUp
	DDR_INT &= ~(1 << PINNR_INT);
	PORT_INT &= ~(1 << PINNR_INT);

	// INT0 als Eingang mit PullUp
	DDR_CLKIN &= ~(1 << PINNR_CLKIN);
	PORT_CLKIN |= (1 << PINNR_CLKIN);
}


/**
* \brief
* Initialisiert die Pins der AD-Eingaenge
*/
void Init_ADPins ( void )
{
	// PCO als Eingang ohne PullUp
	UBATT_DDR &= ~(1 << UAKKU_PINNR);
	UBATT_PORT &= ~(1 << UAKKU_PINNR);

	// PC1 als Eingang ohne PullUp
	UAKKU_DDR &= ~(1 << UAKKU_PINNR);
	UAKKU_PORT &= ~(1 << UAKKU_PINNR);
}


/**
* \brief
* Initialisiert die PinChangeInterrupts
*/
void Init_PCI ( void )
{
	// Enable PCIE 23..16
	PCICR |= (1 << PCIE2);

		// CLK aktivieren
		PCMSK2 |= (1 << PCINT18);

		// SW Next aktivieren
		PCMSK2 |= (1 << PCINT16);

		// SW Mode aktivieren
		PCMSK2 |= (1 << PCINT20);


	// Enable PCIE 15..8
	PCICR |= (1 << PCIE1);

		// SW Light aktivieren
		PCMSK1 |= (1 << PCINT10);

		// SW Set aktivieren
		PCMSK1 |= (1 << PCINT11);
}


/**
* \brief
* Initialisiert die Eingangspins der Tasten
*/
void Init_Tasten ( void )
{
	// Alle als Eingaenge mit PullUp
	TASTER_NEXT_DDR &= ~(1 << TASTER_NEXT_PINNR);
	TASTER_ESC_DDR &= ~(1 << TASTER_ESC_PINNR);
	TASTER_OK_DDR &= ~(1 << TASTER_OK_PINNR);
	TASTER_LIGHT_DDR &= ~(1 << TASTER_LIGHT_PINNR);

	TASTER_NEXT_PORT |= (1 << TASTER_NEXT_PINNR);
	TASTER_ESC_PORT |= (1 << TASTER_ESC_PINNR);
	TASTER_OK_PORT |= (1 << TASTER_OK_PINNR);
	TASTER_LIGHT_PORT |= (1 << TASTER_LIGHT_PINNR);
}


/**
* \brief
* Initialisiert die PWM Ausgangspins
*/
void Init_PWMPins ( void )
{
	// PWM's auf Low (0)
	REDPORT &= ~(1 << REDPINNR);
	GREENPORT &= ~(1 << GREENPINNR);
	BLUEPORT &= ~(1 << BLUEPINNR);
	WHITEPORT &= ~(1 << WHITEPINNR);
	
	// PWM's als Ausgaenge (1)
	REDDDR |= (1 << REDPINNR);
	GREENDDR |= (1 << GREENPINNR);
	BLUEDDR |= (1 << BLUEPINNR);	
	WHITEDDR |= (1 << WHITEPINNR);
}


/**
* \brief
* Initialisiert die Timer fuer die PWM erzeugung
*/
void Init_PWMTimers ( void )
{
	// Timer/Counter 0 initialization
	// Clock source: System Clock
	// Clock value: 125.000 kHz
	// Mode: Fast PWM top=FFh
	// OC0A output: Non-Inverted PWM
	// OC0B output: Non-Inverted PWM
	TCCR0A=0xA3;
	TCCR0B=0x02;
	TCNT0=0x00;
	OCR0A=0x00;
	OCR0B=0x00;


	// Timer/Counter 1 initialization
	// Clock source: System Clock
	// Clock value: 125.000 kHz
	// Mode: Fast PWM top=00FFh
	// OC1A output: Non-Inv.
	// OC1B output: Non-Inv.
	// Noise Canceler: Off
	// Input Capture on Falling Edge
	// Timer 1 Overflow Interrupt: Off
	// Input Capture Interrupt: Off
	// Compare A Match Interrupt: Off
	// Compare B Match Interrupt: Off
	TCCR1A=0xA1;
	TCCR1B=0x0A;
	TCNT1H=0x00;
	TCNT1L=0x00;
	ICR1H=0x00;
	ICR1L=0x00;
	OCR1AH=0x00;
	OCR1AL=0x00;
	OCR1BH=0x00;
	OCR1BL=0x00;
}


/**
* \brief
* Initialisiert den Timer0 Overflow Interrupt fuer das PWM-Signal
*/
void Init_PWMInterrupt ( void )
{
	// Timer0 Overflow Interrupt
	TIMSK0 |= (1 << TOIE0);
}


/**
* \brief
* Schaltet die nicht gebrauchte Peripherie des Prozessors ab
*/
void PowerSave ( void )
{
	power_adc_disable();
	power_spi_disable();
	power_timer0_disable();
	power_timer1_disable();
	power_timer2_disable();
	power_usart0_disable();
}


/**
* \brief
* Intitialisiert den Piepser Pin
*/
void Init_Buzzer ( void )
{
	BUZZERDDR |= (1 << BUZZERPINNR);
	BUZZERPORT |= (1 << BUZZERPINNR);
}
