/*******
 menue.c
********/


//***********
//* Includes
//***********
#include "menue.h"


//***********
//* Variabeln
//***********


/**
* \brief
* Diese Funktion wird immer aufgerufen. Sie zeigt die 1. Ebene falls man sich auf dieser befindet, ansonsten wird zur naechsten Ebene gewechselt
* \note
* Diese Funktion wird immer wieder aufgerufen und ist keine Endlosschleife
*/
void Menue ( void )
{
	unsigned char i;
	
	// Die Tasten duerfen nur ausgewerte werden wenn nicht gerade eine Funktion diese braucht
	if ((FunctionBusy == FALSE) && (AppMode[0] != MODE_STDSCREEN))
	{
		if (OK.Kurz == TRUE)
		{
			OKEntprellen();
			OK.Kurz = FALSE;
			
			IncChar(&AppLayer, 2, 2);
			for (i=AppLayer; i<MODE_DEPTH; i++)
			{
				AppMode[i] = 0;
			}
			Zeit.MinChanged = TRUE;
		}
		
		if (ESC.Kurz == TRUE)
		{
			ESCEntprellen();
			ESC.Kurz = FALSE;
			
			if (AppLayer == 0)
			{
				AppMode[0] = MODE_STDSCREEN;
			}
			DecChar(&AppLayer, 0, 0);
			Zeit.MinChanged = TRUE;
		}
	}
	
	// Next Taste nur auswerten wenn man sich in diesem Layer befindet
	if (AppLayer == 0)
	{
		if (AppMode[0] == MODE_STDSCREEN)
		{
			AppLayer = 0;
			AppMode[1] = 0;
			AppMode[2] = 0;
			OKEntprellen();
			OK.Kurz = FALSE;
		}
		
		if (Next.Kurz == TRUE)
		{
			NextEntprellen();
			Next.Kurz = FALSE;
			
			IncChar(&AppMode[0], MODE_ENTRY_MAX, 0);
			Zeit.MinChanged = TRUE;
		}
	}
	
	switch (AppMode[0])
	{
		// (0,x,x) Zeitanzeige
		//---------------------
		case MODE_STDSCREEN:
			if (Zeit.MinChanged == TRUE)
			{
				Zeit.MinChanged = FALSE;
				LCD_StandardPrompt();
				AppMode[1] = 0;
				AppMode[2] = 0;
			}
			break; //MODE_TIME
			
			
		// (1,0,x) Alarm
		//---------------
		case MODE_ALARM:
			if (AppLayer == 0)
			{
				if (Zeit.MinChanged == TRUE)
				{
					Zeit.MinChanged = FALSE;
					Disp_Clear();
					Disp_SetAddr(0,0);
					Disp_String("Alarm");
				}
			}
			else
			{
				LayerAlarm();
			}
			break; //MODE_ALARM
			
			
		// (2,0,x) Einstellungen
		//-----------------------
		case MODE_SETTINGS:
			if (AppLayer == 0)
			{
				if (Zeit.MinChanged == TRUE)
				{
					Zeit.MinChanged = FALSE;
					Disp_Clear();
					Disp_SetAddr(0,0);
					Disp_String("Einstellungen");
				}
			}
			else
			{
				LayerSettings();
			}
			break; //MODE_SETTINGS
	}
	
	ApplicationBusy = FALSE;
}


/**
* \brief
* Sie zeigt die 2. Ebene falls man sich auf dieser befindet, ansonsten wird zur naechsten Ebene gewechselt
* \note
* Diese Funktion wird immer wieder aufgerufen und ist keine Endlosschleife
*/
void LayerAlarm ( void )
{
	unsigned char Hour;
	unsigned char Min;
	
	// Next Taste nur auswerten wenn man sich in diesem Layer befindet
	if (AppLayer == 1)
	{
		if (Next.Kurz == TRUE)
		{
			NextEntprellen();
			Next.Kurz = FALSE;
			
			IncChar(&AppMode[1], ALARM_ENTRY_MAX, 0);
			Zeit.MinChanged = TRUE;
		}
	}

	switch (AppMode[1])
	{
		// (1,1,0) Alarm On/Off
		//----------------------
		case ALARM_ONOFF:
			
			// Layer 1 (Menue)
			if (AppLayer == 1)
			{
				if (Zeit.MinChanged == TRUE)
				{
					Zeit.MinChanged = FALSE;
					Disp_Clear();
					Disp_SetAddr(0,0);
					Disp_String("Alarm");
					Disp_SetAddr(1,0);
					Disp_String("Ein / Aus");			
				}
			}
			
			// Layer 2 (Funktion)
			if (AppLayer == 2)
			{
				FunctionBusy = TRUE;
				if (LayerAlarm2() == TRUE)
				{
					FunctionBusy = FALSE;
				}
			}
		break; //ALARM_1ONOFF
		
		
		// (1,1,1) Alarm Set
		//-------------------
		case ALARM_SET:
			
			// Layer 1 (Menue)
			if (AppLayer == 1)
			{
				if (Zeit.MinChanged == TRUE)
				{
					Zeit.MinChanged = FALSE;
					Disp_Clear();
					Disp_SetAddr(0,0);
					Disp_String("Alarm");
					Disp_SetAddr(1,0);
					Disp_String("Zeit");
				}
			}
			
			// Layer 2 (Funktion)
			if (AppLayer == 2)
			{
				Hour = Alarm.Hour;
				Min = Alarm.Min;
				FunctionBusy = TRUE;
				SetTime(&Hour, &Min, &SetAlarmBusy);

				if (SetAlarmBusy == FALSE)
				{
					AppMode[2] = 0;
					Alarm.Hour = Hour;
					Alarm.Min = Min;
					WriteAlarmState();
					Disp_Cursor_Off();
					FunctionBusy = FALSE;
				}
			}
		break; //ALARM_1SET
		
		
		// (1,1,2) Alarm Tage
		//--------------------
		case ALARM_DAYS:
			
			// Layer 1 (Menue)
			if (AppLayer == 1)
			{
				if (Zeit.MinChanged == TRUE)
				{
					Zeit.MinChanged = FALSE;
					Disp_Clear();
					Disp_SetAddr(0,0);
					Disp_String("Alarm");
					Disp_SetAddr(1,0);
					Disp_String("Tage");			
				}
			}
			
			// Layer 2 (Funktion)
			if (AppLayer == 2)
			{				
				// Tage eingestellt
				FunctionBusy = TRUE;
				SetAlarmWeekdays(&SetAlarmDaysBusy);
				
				if (SetAlarmDaysBusy == FALSE)
				{
					AppMode[2] = 0;
					AppLayer--;
					Disp_Cursor_Off();
					FunctionBusy = FALSE;
				}
			}
		break; //ALARM_DAYS
	}
}
	

/**
* \brief
* Sie zeigt die 2. Ebene falls man sich auf dieser befindet, ansonsten wird zur naechsten Ebene gewechselt
* \note
* Diese Funktion wird immer wieder aufgerufen und ist keine Endlosschleife
*/
void LayerSettings ( void )
{
	unsigned char Hour;
	unsigned char Min;
	unsigned char Day;
	unsigned char Month;
	unsigned char Year;
	unsigned char Weekday;
	
	// Next Taste nur auswerten wenn man sich in diesem Layer befindet
	if (AppLayer == 1)
	{
		if (Next.Kurz == TRUE)
		{
			NextEntprellen();
			Next.Kurz = FALSE;
			
			IncChar(&AppMode[1], SETTINGS_ENTRY_MAX, 0);
			Zeit.MinChanged = TRUE;
		}
	}


	switch (AppMode[1])
	{
		// (2,1,0) Zeit einstellen
		//-------------------------								
		case SETTINGS_TIME:

			// Layer 1 (Menue)
			if (AppLayer == 1)
			{
				if (Zeit.MinChanged == TRUE)
				{
					Zeit.MinChanged = FALSE;
					Disp_Clear();
					Disp_SetAddr(0,0);
					Disp_String("Einstellungen");
					Disp_SetAddr(1,0);
					Disp_String("Zeit einstellen");
				}
			}

			// Layer 2 (Funktion)
			if (AppLayer == 2)
			{
				Hour = RTC.Hour;
				Min = RTC.Min;
				SetTime(&Hour, &Min, &SetTimeBusy);
				FunctionBusy = TRUE;

				if (SetTimeBusy == FALSE)
				{
					AppMode[2] = 0;
					PCF8563_SetTime(0, Min, Hour, RTC.Day, RTC.Weekday, RTC.Month, RTC.Year);
					PCF8563_GetTime();
					Disp_Cursor_Off();
					FunctionBusy = FALSE;
				}
			}
			break; //SETTINGS_TIME
			

		// (2,1,1) Datum einstellen
		//--------------------------
		case SETTINGS_DATE:
		
			// Layer 1 (Menue)
			if (AppLayer == 1)
			{
				if (Zeit.MinChanged == TRUE)
				{
					Zeit.MinChanged = FALSE;
					Disp_Clear();
					Disp_SetAddr(0,0);
					Disp_String("Einstellungen");
					Disp_SetAddr(1,0);
					Disp_String("Datum einstellen");
				}
			}

			// Layer 2 (Funktion)
			if (AppLayer == 2)
			{
				Day = RTC.Day;
				Month = RTC.Month;
				Year = RTC.Year;
				Weekday = RTC.Weekday;
				SetDate(&Year, &Month, &Day, &Weekday, &SetDateBusy);
				FunctionBusy = TRUE;

				if (SetDateBusy == FALSE)
				{
					AppMode[2] = 0;
					PCF8563_SetTime(0, RTC.Min, RTC.Hour, Day, Weekday, Month, Year);
					PCF8563_GetTime();
					Disp_Cursor_Off();
					FunctionBusy = FALSE;
				}
			}
			break; //SETTINGS_DATE


		// (2,1,2) Spannungen ansehen
		//----------------------------
		case SETTINGS_VOLTAGES:
			
			// Layer 1 (Menue)
			if (AppLayer == 1)
			{
				if (Zeit.MinChanged == TRUE)
				{
					Zeit.MinChanged = FALSE;
					Disp_Clear();
					Disp_SetAddr(0,0);
					Disp_String("Einstellungen");
					Disp_SetAddr(1,0);
					Disp_String("Spannungen anz.");
				}
			}
			
			// Layer 2 (Funktion)
			if (AppLayer == 2)
			{
				if (Zeit.SecChanged == TRUE)
				{
					Zeit.SecChanged = FALSE;
					Disp_AdjustContrast();
					Disp_Clear();
					Disp_SetAddr(0,0);
					Disp_Integer(Voltage_Batt);
					Disp_SetAddr(0,5);
					Disp_String("mV");
					Disp_SetAddr(1,0);
					Disp_Integer(Voltage_Accu);
					Disp_SetAddr(1,5);
					Disp_String("mV");
				}
			}

			break; //SETTINGS_DATE
	}
}


/**
* \brief
* Diese Funktion wird immer aufgerufen. Sie zeigt die 3. Ebene.
* \note
* Diese Funktion wird immer wieder aufgerufen und ist keine Endlosschleife
* \return
* FALSE solange noch nicht abgeschlossen mit ESC oder OK
*/
unsigned char LayerAlarm2 ( void )
{
	unsigned char State = FALSE;
	unsigned char RetVal = FALSE;
	
	if (Next.Kurz == TRUE)
	{
		NextEntprellen();
		Next.Kurz = FALSE;
		
		IncChar(&AppMode[2], SETALARM_ONOFF_MAX, 0);
		Zeit.MinChanged = TRUE;
	}
		
	switch (AppMode[2])
	{
		// (1,1,0) Alarm Einschalten
		//---------------------------
		case SETALARM_ON:
			if (Zeit.MinChanged == TRUE)
			{
				Zeit.MinChanged = FALSE;
				Disp_Clear();
				Disp_SetAddr(0,0);
				Disp_String("Alarm");
				Disp_SetAddr(1,0);
				Disp_String("Ein");
			}
			State = TRUE;
			break; //SETALARM_ON

		// (1,1,0) Alarm Ausschalten
		//---------------------------
		case SETALARM_OFF:
			if (Zeit.MinChanged == TRUE)
			{
				Zeit.MinChanged = FALSE;
				Disp_Clear();
				Disp_SetAddr(0,0);
				Disp_String("Alarm");
				Disp_SetAddr(1,0);
				Disp_String("Aus");
			}
			State = FALSE;
			break; //SETALARM_OFF
	}

	// Einstellung speichern und ein Layer zurueck im Menue
	if (OK.Kurz == TRUE)
	{
		OKEntprellen();
		OK.Kurz = FALSE;
		AppLayer--;
		Alarm.State = State;
		RetVal = TRUE;
		Zeit.MinChanged = TRUE;
		
		WriteAlarmState();
		
		TaskHandler_AddTask(CheckAlarm);
	}
	
	// ein Layer zurueck im Menue
	if (ESC.Kurz == TRUE)
	{
		ESCEntprellen();
		ESC.Kurz = FALSE;
		AppLayer--;
		RetVal = TRUE;
		Zeit.MinChanged = TRUE;
	}
	
	return RetVal;
}
