/**
* \file
* scheduler.h
* \brief
* Einfacher TaskScheduler
* \note
* Flash Speicherverbrauch: 418 Byte
* \version
* v. 0.1 Erste Version
* \date
* Create: 24.01.2009 <br>
* Modify: 15.02.2009
* \author
* Philipp Kaelin
*/
#ifndef _TASKSCHEDULER_H_
	#define _TASKSCHEDULER_H_

	//**********
	//* Include 
	//**********
	#include <avr/interrupt.h>
	#include "application.h"

	
	//*********
	//* Config
	//*********
	/** Anzahl der Tasks, die gespeichert werden koennen */
	#define TASKSCHEDULER_SIZE						16
	
	/** Die folgende Zeile auskommentieren wenn die Interrupts
		waehrend dem Sortieren nicht ausgeschaltete werden sollen.
		Dies ist um zu verhindern dass ein neuer Task hinzugefuegt
		wird waehrend dem sortieren */
	#define DISABLE_INTERRUPT_DURING_SORT
	
	/** Wenn TRUE dann wird ein Reset ausgefuehrt wenn ein Task 
		hinzugefuegt werden soll, die Queue aber voll ist. 
		Einfacher Watchdog */
	#define SCHEDULER_RESET_IF_TASK_QUEUE_FULL		TRUE
	
	//**********
	//* Defines
	//**********
	/** Task wurde erfolgreich zur Queue hinzugefuegt */
	#define TASK_ADDED								1
	/** Die Task Queue ist voll */
	#define TASK_QUEUE_FULL							2
	/** Der Task wurde erfolgreich entfernt */
	#define TASK_REMOVED							3
	/** Der Task konnte nicht entfern werden */
	#define	TASK_NOT_FOUND							4
	
	/** void Pointer fuer zugriff auf Funktionen */
	#ifndef code
		#define code
		typedef void (code *pFunction) ( void );
	#endif
	
	#ifndef TRUE
		#define TRUE								1
	#endif
	
	#ifndef FALSE
		#define FALSE								0
	#endif


	//************************
	//* Function Declarations
	//************************
	void TaskScheduler_Init ( void );
	
	void TaskScheduler_TimerTick ( void );
	
	unsigned char TaskScheduler_AddTask ( pFunction TaskToAdd, unsigned int uiTimeUntilExecute );

	unsigned char TaskScheduler_RemoveTask ( pFunction TaskToRemove );
	
	void TaskScheduler_SortTasks ( void );
	
#endif
