/**
* \file
* task_handler.h
* \brief
* Einfacher TaskHandler ohne Prioritaet
* \note
* Flash Speicherverbrauch: 216 Byte
* \version
* v. 0.3 TaskHandler_AddTaskIfNotExist hinzugefuegt <br>
* v. 0.2 Wenn die Queue Voll ist kann ein Reset ausgefuehrt werden <br>
* v. 0.1 Erste Version
* \date
* Create: 13.11.2008 <br>
* Modify: 29.01.2009
* \author
* Philipp Kaelin
*/
#ifndef _TASKHANDLER_H_
	#define _TASKHANDLER_H_

	//**********
	//* Include 
	//**********
	#include "application.h"


	//*********
	//* Config
	//*********
	/** Anzahl der Tasks, die gespeichert werden koennen */
	#define TASKHANDLER_SIZE			16
	
	/** Wenn TRUE dann wird ein Reset ausgefuehrt wenn ein Task 
		hinzugefuegt werden soll, die Queue aber voll ist. 
		Einfacher Watchdog */
	#define RESET_IF_TASK_QUEUE_FULL	TRUE
	

	//**********
	//* Defines
	//**********
	/** Task wurde erfolgreich zur Queue hinzugefuegt */
	#define TASK_ADDED					1
	/** Die Task Queue ist voll */
	#define TASK_QUEUE_FULL				2
	/** Keine Tasks mehr in der Queue vorhanden */
	#define TASK_QUEUE_EMPTY			3
	/** Es sind noch einer oder mehrere Tasks vorhanden */
	#define TASK_QUEUE_PRESENT			4
	/** Task ist schon in der Queue und wurde nicht hinzugefuegt */
	#define TASK_FOUND					5
	
	/** void Pointer fuer Funktionen */
	#ifndef code
		#define code
		typedef void (code *pFunction) ( void );
	#endif

	#ifndef FALSE
		#define FALSE					0
	#endif

	#ifndef TRUE
		#define TRUE					1
	#endif


	//************************
	//* Function Declarations
	//************************

	unsigned char TaskHandler_HandleEvents ( void );

	unsigned char TaskHandler_AddTask ( pFunction TaskToAdd );
	
	unsigned char TaskHandler_AddTaskIfNotExist ( pFunction TaskToAdd );

	void TaskHandler_IncPos ( unsigned char *pAddress );


#endif
